"""Connection class implementation."""
from enum import Enum

import requests
from pyaml_env import parse_config

USERNAME = 'USERNAME'
PASSWORD = 'PASSWORD'

requests.packages.urllib3.disable_warnings(
    requests.packages.urllib3.exceptions.InsecureRequestWarning)


class IOType(Enum):
    """Enum type for input types."""

    TEXT = "text"
    FILE = "file"
    FILE_ID = "file_id"

    def __str__(self):
        """To string for IOType Enum."""
        return self.name.lower()

    def __repr__(self):
        """String representation for IOType Enum."""
        return str(self)

    @staticmethod
    def argparse(s):
        """Argument parsing for argparser type."""
        try:
            return IOType[s.upper()]
        except KeyError:
            return s


class Connection:
    """Implementation of Connection class."""

    BASE_API_URL = "https://services-test.clarin-pl.eu/api/v1"
    API_ENDPOINTS = {
        "login": "/oauth/login",
        "files": "/files/",
        "tasks": {"start": "/tasks/",
                  "run": "/tasks/run/",
                  "progress": "/tasks/",
                  "sent": "/tasks/sent/",
                  "status": "/tasks/users/",
                  "cancel": "/tasks/cancel/"}
    }

    def __init__(self, username='', password='',
                 baseurl='', config_file='') -> None:
        """Configure connection authorization with api.

        :param username: e-science username
        :type username: str
        :param password: e-science user password
        :type password: str
        :param baseurl: base api url if different from default
        :type baseurl: str
        :param config_file: path of user config file with username and password
        :type config_file: str
        """
        self.token = None
        self.base_url = baseurl if baseurl else Connection.BASE_API_URL
        if username and password:
            self.token = self.__get_api_token(username, password)
        elif config_file:
            config = parse_config(config_file)
            if USERNAME in config and PASSWORD in config:
                self.token = self.__get_api_token(config[USERNAME],
                                                  config[PASSWORD])

        if self.token is None:
            raise Exception('Authorization Error')

    def __get_api_token(self, username: str, password: str):
        r = requests.post(
            url=f"{self.base_url}{Connection.API_ENDPOINTS['login']}",
            json={
                "username": username,
                "password": password
            },
            verify=False
        )
        if r.status_code == 200:
            data = r.json()
            return data["access_token"]
